@@ -116,15 +116,17 @@
 // Polling process
 void UdpAccelPoll(UDP_ACCEL *a)
 {
-	UCHAR *tmp = a->TmpBuf;
 	IP nat_t_ip;
 	UINT num_ignore_errors = 0;
+	UCHAR *tmp;
 	// Validate arguments
 	if (a == NULL)
 	{
 		return;
 	}
 
+	tmp = a->TmpBuf;
+
 	Lock(a->NatT_Lock);
 	{
 		Copy(&nat_t_ip, &a->NatT_IP, sizeof(IP));
